package com.webstersmalley.iplayer.feed;

/**
 *    Copyright 2009 Webster Smalley
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */


import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashSet;
import java.util.Set;

import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathConstants;
import javax.xml.xpath.XPathExpression;
import javax.xml.xpath.XPathExpressionException;
import javax.xml.xpath.XPathFactory;

import org.w3c.dom.DOMException;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

public class ProgramXMLFactory implements ProgramFactory {
	private final XPath xpath = XPathFactory.newInstance().newXPath();

	private XPathExpression titleExpr;
	private XPathExpression idExpr;
	private XPathExpression updatedExpr;
	private XPathExpression locationExpr;
	private XPathExpression categoriesExpr;
	private XPathExpression descriptionExpr;
	
	// 2009-03-05T19:17:42Z
	private SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-ddHH:mm:ss");
	
	public ProgramXMLFactory() {
		try {
			titleExpr = xpath.compile("./title");
			idExpr = xpath.compile("./id");
			updatedExpr = xpath.compile("./updated");
			locationExpr = xpath.compile("./link[@rel='self']");
			categoriesExpr = xpath.compile("./category");
			descriptionExpr = xpath.compile("./content[@type='html']");
		} catch (Exception e) {
			throw new RuntimeException("Error instantiating factory", e);
		}
	}

	private String getNodeText(Element element, XPathExpression expression) throws DOMException, XPathExpressionException {
		return ((Node)expression.evaluate(element, XPathConstants.NODE)).getTextContent();
	}
	
	private Date getNodeDate(Element element, XPathExpression expression) throws DOMException, XPathExpressionException, ParseException {
		String dateString = getNodeText(element, expression);
		if (dateString.length() != 20) {
			return null;
		}
		dateString = dateString.substring(0, 10) + dateString.substring(11, 19);
		return sdf.parse(dateString);
	}
	
	private String getNodeAttribute(Element element, XPathExpression expression, String attribute) throws XPathExpressionException {
		Element node = (Element)expression.evaluate(element, XPathConstants.NODE);
		return node.getAttribute(attribute);
	}
	
	private Set<String> getNodeAttributeSet(Element element, XPathExpression expression, String attribute) throws XPathExpressionException {
		Set<String> returnSet = new HashSet<String>();
		NodeList nodes = (NodeList)expression.evaluate(element, XPathConstants.NODESET);
		for (int i = 0; i < nodes.getLength(); i++) {
			returnSet.add(((Element)nodes.item(i)).getAttribute(attribute));
		}
		return returnSet;
	}
	
	private String getDescription(Element element) throws DOMException, XPathExpressionException {
		String html = getNodeText(element, descriptionExpr);
		html.replaceAll("&lt;", "<").replaceAll("&gt;", ">");
		int lastOpenP = -1;
		while (html.indexOf("<p>", lastOpenP) != -1) {
			lastOpenP = html.indexOf("<p>", lastOpenP) + "<p>".length();
		}
		int lastCloseP = -1;
		while (html.indexOf("</p>", lastCloseP + 1) != -1) {
			lastCloseP = html.indexOf("</p>", lastCloseP + 1);
		}
		return html.substring(lastOpenP + "<p>".length(), lastCloseP).trim();
	}
	
	public Program getProgramFromElement(Element element) {
		try {
			Program program = new Program();
			program.setTitle(getNodeText(element, titleExpr));
			program.setId(getNodeText(element, idExpr));
			program.setUpdated(getNodeDate(element, updatedExpr));
			program.setLocation(getNodeAttribute(element, locationExpr, "href"));
			program.setCategories(getNodeAttributeSet(element, categoriesExpr, "term"));
			program.setDescription(getDescription(element));
			return program;
		} catch (Exception e) {
			throw new RuntimeException("Error instantiating factory", e);
		}
	}
}
