/**
 *    Copyright 2009 Webster Smalley
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */
package com.webstersmalley.iplayer.mp3;

import java.io.BufferedReader;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.Set;

import org.apache.log4j.Logger;
import org.blinkenlights.jid3.ID3Exception;
import org.blinkenlights.jid3.ID3Tag;
import org.blinkenlights.jid3.MP3File;
import org.blinkenlights.jid3.MediaFile;
import org.blinkenlights.jid3.v2.ID3V2Tag;
import org.blinkenlights.jid3.v2.ID3V2_3_0Tag;

import com.webstersmalley.iplayer.informer.ProgramInformation;
import com.webstersmalley.iplayer.organiser.RadioProgramBean;
import com.webstersmalley.musiclibrary.model.FileSystemMusicFileFactory;
import com.webstersmalley.musiclibrary.model.MusicFile;

/**
 * @author Matthew Smalley
 * 
 */
public class MP3Utils {
	private static final String COMMAND_LINE_TEMPLATE = "%1$s --nohist --preset \"%2$s\" \"%3$s\" \"%4$s\"";
	private String mp3Encoder;
	private FileSystemMusicFileFactory musicFileFactory = new FileSystemMusicFileFactory();
	private FileSystemUtils fileSystemUtils = new FileSystemUtils();

	private Logger logger = Logger.getLogger(getClass());

	public void setMp3Encoder(String mp3Encoder) {
		this.mp3Encoder = mp3Encoder;
	}

	public ID3V2Tag getID3V2Tags(String filename) {
		try {
			MediaFile mediaFile = new MP3File(new File(filename));
			ID3Tag[] tags = mediaFile.getTags();
			for (ID3Tag tag : tags) {
				if (tag instanceof ID3V2Tag) {
					return (ID3V2Tag) tag;
				}
			}
		} catch (ID3Exception e) {
			throw new RuntimeException("Error reading tags", e);
		}
		return null;
	}

	public String dumpAllTags(String filename) {
		StringBuffer sb = new StringBuffer();
		try {
			MediaFile mediaFile = new MP3File(new File(filename));

			ID3Tag[] tags = mediaFile.getTags();

			for (ID3Tag tag : tags) {
				sb.append(tag.getClass());
				sb.append("\n");
				sb.append(tag);

				sb.append("\n");
			}

		} catch (ID3Exception e) {
			throw new RuntimeException("Error reading tags", e);
		}

		return sb.toString();
	}

	public void reencodeFolder(String inputFolder, String outputFolder, String preset) {
		logger.info("Re-encoding folder: " + inputFolder + " into " + outputFolder + ", using preset " + preset);
		if (inputFolder == null || outputFolder == null || preset == null) {
			throw new RuntimeException("Null parameters not allowed");
		}
		if (!inputFolder.endsWith(File.separator)) {
			inputFolder += File.separator;
		}
		if (!outputFolder.endsWith(File.separator)) {
			outputFolder += File.separator;
		}

		Set<MusicFile> inputFiles = musicFileFactory.getMusicFiles(inputFolder);
		for (MusicFile musicFile : inputFiles) {
			String inputFilename = inputFolder + musicFile.getFilename();
			String outputFilename = outputFolder + musicFile.getFilename();
			logger.info("Processing file: " + inputFilename);
			fileSystemUtils.checkFolder(new File(outputFilename).getParentFile().getAbsolutePath());
			if (fileSystemUtils.destinationFileNewer(inputFilename, outputFilename)) {
				logger.info("Destination file exists + is newer, skipping");
			} else {
				logger.info("Running: " + inputFilename + ", " + outputFilename);
				reencodeMP3(inputFilename, outputFilename, preset);
			}
		}
	}

	public void reencodeMP3(String inputFilename, String outputFilename, String preset) {
		reencodeMP3(inputFilename, outputFilename, preset, true);
	}

	public void reencodeMP3(String inputFilename, String outputFilename, String preset, boolean retag) {

		try {
			File outputFolder = new File(outputFilename).getParentFile();
			if (!outputFolder.exists()) {
				if (!outputFolder.mkdirs()) {
					throw new RuntimeException("Error creating output folder");
				}
			}
			String commandLine = String.format(COMMAND_LINE_TEMPLATE, mp3Encoder, preset, inputFilename, outputFilename).replaceAll("\\\\", "/");

			logger.debug("Running command: " + commandLine);

			Process p = Runtime.getRuntime().exec(commandLine);

			StreamConsumer stdout = new StreamConsumer(p.getInputStream(), true);
			StreamConsumer error = new StreamConsumer(p.getErrorStream(), false);

			new Thread(stdout, "stdout reader").start();
			new Thread(error, "error reader").start();

			p.waitFor();
			int exitValue = p.exitValue();
			if (exitValue != 0) {
				throw new RuntimeException("MP3 Encoder exited with code: " + exitValue);
			}
			if (retag) {
				copyTags(inputFilename, outputFilename);
			}
		} catch (Exception e) {
			logger.error("Error running mp3 encoder", e);
			throw new RuntimeException("Error running mp3 encoder", e);
		}
	}

	private void copyTags(String inputFilename, String outputFilename) {
		ID3V2Tag inputTags = getID3V2Tags(inputFilename);
		MediaFile mediaFile = new MP3File(new File(outputFilename));
		mediaFile.setID3Tag(inputTags);
		try {
			mediaFile.sync();
		} catch (ID3Exception e) {
			throw new RuntimeException("Error writing tags to: " + outputFilename, e);
		}
	}

	private class StreamConsumer implements Runnable {
		private Logger logger = Logger.getLogger(getClass());
		private InputStream stream;
		private boolean log;

		public StreamConsumer(InputStream stream, boolean log) {
			this.stream = stream;
			this.log = log;
		}

		public void run() {
			BufferedReader br = new BufferedReader(new InputStreamReader(stream));
			try {
				for (String line; (line = br.readLine()) != null;) {
					if (log) {
						logger.info(line);
					}
				}
			} catch (IOException e) {
				logger.error("Error reading stream", e);
			}
		}
	}

	public void retagMP3(String filename, RadioProgramBean program) {
		try {
			ID3V2_3_0Tag tag = new ID3V2_3_0Tag();
			tag.setTitle(program.getEpisodeTitle());
			tag.setAlbum(program.getSeriesTitle());
			tag.setArtist(program.getArtist());

			MediaFile mediaFile = new MP3File(new File(filename));
			mediaFile.setID3Tag(tag);
			mediaFile.sync();
		} catch (Exception e) {
			logger.error("Error writing mp3 tags", e);
			throw new RuntimeException("Error writing mp3 tags", e);
		}
	}

	public void retagMP3(String filename, ProgramInformation prog) {
		try {
			ID3V2_3_0Tag tag = new ID3V2_3_0Tag();
			tag.setTitle(prog.getProgramName());
			tag.setAlbum(prog.getProgramSetName());
			tag.setArtist(prog.getAuthor());

			MediaFile mediaFile = new MP3File(new File(filename));
			mediaFile.setID3Tag(tag);
			mediaFile.sync();
		} catch (Exception e) {
			logger.error("Error writing mp3 tags", e);
			throw new RuntimeException("Error writing mp3 tags", e);
		}
	}

}
