package com.webstersmalley.iplayer.organiser;

/**
 *    Copyright 2009 Webster Smalley
 *
 *   Licensed under the Apache License, Version 2.0 (the "License");
 *   you may not use this file except in compliance with the License.
 *   You may obtain a copy of the License at
 *
 *       http://www.apache.org/licenses/LICENSE-2.0
 *
 *   Unless required by applicable law or agreed to in writing, software
 *   distributed under the License is distributed on an "AS IS" BASIS,
 *   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *   See the License for the specific language governing permissions and
 *   limitations under the License.
 *
 */

import java.io.File;
import java.io.FilenameFilter;

import org.apache.log4j.Logger;
import org.blinkenlights.jid3.ID3Exception;
import org.blinkenlights.jid3.ID3Tag;
import org.blinkenlights.jid3.MP3File;
import org.blinkenlights.jid3.MediaFile;
import org.blinkenlights.jid3.v1.ID3V1_0Tag;
import org.blinkenlights.jid3.v1.ID3V1_1Tag;
import org.blinkenlights.jid3.v1.ID3V1Tag.Genre;
import org.blinkenlights.jid3.v2.ID3V2Tag;
import org.blinkenlights.jid3.v2.ID3V2_3_0Tag;
import org.springframework.context.ApplicationContext;
import org.springframework.context.support.ClassPathXmlApplicationContext;

import com.webstersmalley.iplayer.mp3.MP3Utils;

public class Retagger {
	protected class MP3FilenameFilter implements FilenameFilter {

		public boolean accept(File dir, String name) {
			return name.toLowerCase().matches(".*\\.mp3");
		}

	}

	private Logger logger = Logger.getLogger(getClass());

	private String inputFolder;
	private String outputFolder;
	private RadioProgramBeanFactory beanFactory;

	public RadioProgramBeanFactory getBeanFactory() {
		return beanFactory;
	}

	public void setBeanFactory(RadioProgramBeanFactory beanFactory) {
		this.beanFactory = beanFactory;
	}

	public String getInputFolder() {
		return inputFolder;
	}

	public void setInputFolder(String inputFolder) {
		this.inputFolder = inputFolder;
	}

	public String getOutputFolder() {
		return outputFolder;
	}

	public void setOutputFolder(String outputFolder) {
		this.outputFolder = outputFolder;
	}

	private void rewriteID3Tags(File mp3file, RadioProgramBean bean) {

		logger.info("Title: " + bean.getEpisodeTitle());
		logger.info("Info: " + bean);
		MediaFile mediaFile = new MP3File(mp3file);
		boolean tagsExist = false;
		try {
			ID3Tag[] tags = mediaFile.getTags();
			mediaFile.removeTags();
			for (ID3Tag tag : tags) {
				if (tag instanceof ID3V1_0Tag) {
					logger.info("Rewriting id3 1.0 tag");
					ID3V1_0Tag v10tag = (ID3V1_0Tag) tag;
					v10tag.setTitle(bean.getEpisodeTitle());
					v10tag.setGenre(Genre.Vocal);
					v10tag.setAlbum(bean.getSeriesTitle());
					v10tag.setArtist(bean.getArtist());
					mediaFile.setID3Tag(v10tag);
					tagsExist = true;
				} else if (tag instanceof ID3V1_1Tag) {
					logger.info("Rewriting id3 1.1 tag");
					ID3V1_1Tag v11tag = (ID3V1_1Tag) tag;
					v11tag.setTitle(bean.getEpisodeTitle());
					v11tag.setGenre(Genre.Vocal);
					v11tag.setAlbum(bean.getSeriesTitle());
					v11tag.setArtist(bean.getArtist());
					mediaFile.setID3Tag(v11tag);
					tagsExist = true;
				} else if (tag instanceof ID3V2Tag) {
					logger.info("Rewriting id3 2 tag");
					ID3V2Tag v2tag = (ID3V2Tag) tag;
					v2tag.setTitle(bean.getEpisodeTitle());
					v2tag.setGenre("Audio book/play");
					v2tag.setAlbum(bean.getSeriesTitle());
					v2tag.setArtist(bean.getArtist());
					mediaFile.setID3Tag(v2tag);
					tagsExist = true;
				} else if (tag instanceof ID3V2_3_0Tag) {
					logger.info("Rewriting id3 2.3.0 tag");
					ID3V2_3_0Tag v230tag = (ID3V2_3_0Tag) tag;
					v230tag.setTitle(bean.getEpisodeTitle());
					v230tag.setGenre("Audio book/play");
					v230tag.setAlbum(bean.getSeriesTitle());
					v230tag.setArtist(bean.getArtist());
					mediaFile.setID3Tag(v230tag);
					tagsExist = true;
				}
			}
			if (!tagsExist) {
				logger.info("Creating new tag");
				ID3V2_3_0Tag tag = new ID3V2_3_0Tag();
				tag.setTitle(bean.getEpisodeTitle());
				mediaFile.setID3Tag(tag);
			}
			mediaFile.sync();
		} catch (ID3Exception e) {
			logger.error("Error updating tags: ", e);
			throw new RuntimeException("Error updating tags: " + e.getMessage());
		}
	}

	private void moveFile(File file, RadioProgramBean bean) {
		String folder = outputFolder + File.separator + bean.getFolder();
		String filename = folder + File.separator + bean.getFilename();

		File folderFile = new File(folder);
		if (!folderFile.exists()) {
			if (!folderFile.mkdirs()) {
				throw new RuntimeException("Error creating folder: " + folder);
			}
		} else {
			if (!folderFile.isDirectory()) {
				throw new RuntimeException("Error creating folder: " + folder + " - it already exists but is not a directory...!");
			}
		}
		file.renameTo(new File(filename));
	}

	private void processFile(File file) {
		RadioProgramBean bean = beanFactory.getBeanFromFile(file.getName());
		rewriteID3Tags(file, bean);
		moveFile(file, bean);
	}

	public void go() {
		logger.info("Opening folder: " + inputFolder);
		File folder = new File(inputFolder);
		if (!folder.isDirectory()) {
			throw new RuntimeException("Given folder name is not a folder or does not exist");
		}
		File[] files = folder.listFiles(new MP3FilenameFilter());

		for (File file : files) {
			processFile(file);
		}
	}

	public static void main(String[] args) {
		ApplicationContext ac = new ClassPathXmlApplicationContext("beans-production.xml");
		Retagger retagger = (Retagger) ac.getBean("retagger");
		retagger.go();
	}
}
